<!DOCTYPE html>
<html>
<head>
        <title>Order and Address MultiView</title>
</head>
<body>

<form action="." method="post">

{{ order_form }}

<input type="submit" value="Submit" />

</form>

<form action="." method="post">

{{ address_form }}

<input type="submit" value="Submit" />

</form>

</body>
</html>